/* -*- Mode: C++ -*-
 * $Id: FCFooCoach.cc,v 1.1 2001/11/15 16:50:13 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooCoach.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2001-11-15
 * Last modified by $Author: frehe $ $Date: 2001/11/15 16:50:13 $
 *
 *
 * HISTORY
 * $Log: FCFooCoach.cc,v $
 * Revision 1.1  2001/11/15 16:50:13  frehe
 * Added a coach agent
 *
 */

/**
 * @file FCFooCoach.cc
 * Implementation file for the example coach FCFooCoach.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2001/11/15 16:50:13 $
 */


#include <fstream>

#include "framework_compiler_directives.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "FrameworkAgent.h"
#include "FCFooCoachDecision.h"
#include "LogStream.h"


RS_BEGIN_NAMESPACE


#if defined(RS_USE_ERROR) && !defined(RS_LOG_ERROR)
extern void rs_warning(const std::string& str,
		       const std::string& file,
		       int line)
{
  std::cerr << "WARNING: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}

extern void rs_error(const std::string& str,
		     const std::string& file,
		     int line)
{
  std::cerr << "ERROR: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}

extern void rs_fatal(const std::string& str,
		     const std::string& file,
		     int line)
{
  std::cerr << "FATAL: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
  abort();
}
#endif

#if !defined(RS_USE_EXCEPTION) && !defined(RS_LOG_EXCEPTION)
extern void rs_throw(const std::string& exception,
		     const std::string& msg,
		     const std::string& file,
		     int line)
{
  std::cerr << "RAISED: " << exception << "(\"" << msg << "\")"
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}
#endif


RS_END_NAMESPACE


int main(int argc, char** argv)
{
#ifdef RS_USE_NAMESPACE
  using namespace RS_NAMESPACE;
#endif

  std::ofstream logFile("coach_logfile.txt");
  std::ofstream logFile2("coach_commands.txt");
  std::ofstream logFile3("coach_sensor_data.txt");
  
#ifdef RS_USE_LOG
  LogStream::instance()->addLogArea(LA_ROBOSOC, LA_TOP, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_WARNING, LA_ROBOSOC, true, &logFile, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_ERROR, LA_ROBOSOC, true, &logFile, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_FATAL, LA_ROBOSOC, true, &logFile, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_SERVER_COMMANDS, LA_ROBOSOC, true, &logFile2);
  LogStream::instance()->addLogArea(LA_SENSOR_DATA, LA_ROBOSOC, true, &logFile3);
  LogStream::instance()->addLogArea(LA_CONTROLLER, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_ACTUATORS, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_SENSORS, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_SENSOR_INTERFACE, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_COACH_VIEW, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_DECISION, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_FRAMEWORK_AGENT, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_AGENT, LA_ROBOSOC, true, &logFile);
#endif

  try {

    std::cerr << "Starting the parameter view...\n";
    ParameterView parameterView(PARAMETER_VIEW_ID,
				argc, argv);
    ViewManager::instance()->addView(&parameterView, true);
    
    std::cerr << "Starting coach decision...\n";
    FCFooCoachDecision decision;

    std::cerr << "Initializing the coach agent...\n";
    FrameworkAgent agent(&decision, &parameterView, true);

    std::cerr << "Starting the coach agent...\n";
    agent.startAsCoach();
    
  } catch (RSException &excpt) {
    std::cerr << "Caught a RoboSoc exception: " << excpt.What() << std::endl;
    return 1;
  } catch (std::exception &excpt) {
    std::cerr << "Caught a standard exception: " << excpt.what() << std::endl;
    return 1;
  } catch (...) {
    std::cerr << "Caught an unknown exception: " << std::endl;
    return 1;
  }
  
  return 0;
  
}
