/* -*- Mode: C++ -*-
 * $Id: IsOffside.h,v 1.3 2001/10/03 19:43:39 aip5 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsOffside.h
 * Unitname: Framework/predicate
 * $Revision: 1.3 $
 * Created by: David Rosen 2001-09-30
 * Last modified by $Author: aip5 $Date: 2001/10/03 19:43:39 $
 *
 *
 * HISTORY
 *
 * $Log: IsOffside.h,v $
 * Revision 1.3  2001/10/03 19:43:39  aip5
 * Ganska otestade, men bade offside och markering verkar fungera lite
 *
 * Revision 1.2  2001/10/01 16:08:08  aip5
 * OBS: skriver ut lite nu
 *
 * Revision 1.1  2001/10/01 12:08:08  aip5
 * Berakning av offside for bagge lagen, byggande pa WorldStateView, kompilerar med helt otestad
 *
 *
 */

/**
 * @file IsOffside.h
 * Include file for the predicate class IsOffside.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: aip5 $
 * $Date: 2001/10/03 19:43:39 $
 */

#ifndef ISOFFSIDE_H
#define ISOFFSIDE_H

#include "Predicate.h"
#include "PlayerState.h"

RS_BEGIN_NAMESPACE

class WorldStateView;
class ParameterView;

/**
 * @class IsOffside IsOffside.h
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: aip5 $
 * $Date: 2001/10/03 19:43:39 $
 */
class IsOffside : public Predicate {
public:
  IsOffside (int teamname, const PlayerState &player,
	     bool assumeGoalieIsNearGoal = true);
  IsOffside(const IsOffside& obj);
  virtual ~IsOffside();
  IsOffside& operator= (const IsOffside& obj);
  
  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();

  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();
  virtual void update (const UInt32);
private:
  WorldStateView* worldStateView;
  ParameterView* parameterView;
  int team;
  PlayerState player;
  bool goalieNearGoal;
};


RS_END_NAMESPACE

#endif

