#ifndef FFMOVETOPOSITION_H
#define FFMOVETOPOSITION_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"
#include "TVector.h"
#include "ParameterView.h"
#include "WorldStateView.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class FFMoveToPosition FFMoveToPosition.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: andbo928 $
 * $Date: 2001/09/28 14:11:57 $
 */

class FFMoveToPosition : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  FFMoveToPosition();

  void setPosition(const TVector& pos);
  void setDistance(const double r);

  /** Destructor should clean up after the skill. */
  virtual ~FFMoveToPosition();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  void setLookAtBall(bool enable);

  enum MoveSpeed
  {
    msVeryFast,	// Tillt stamina att hamna ca 10% under grnsen,
		// kanske?
    msFast,	// S fort som mjligt men inte under stamina-grnsen.
		// Dvs som det r nu.
    msNormal,	// Hll stamina ovanfr ca 110% av grnsen, eller ngt
		// liknande.
    msKeepStamina,
		// Anvnd endast s mycket stamina som du fr varje
		// cykel. Det spelar ingen roll om man springer sttvis
		// eller inte, dvs om man dashar med 2 ggr av den stamina
		// man fr och gr det varannan cykel.
    msGainStamina,
		// Anvnd endast hlften av den stamina man fr
		// varje cykel. Jag vet inte hur lngsamt det r,
		// kanske fr lngsamt?
  };
  void setSpeed(const MoveSpeed speed);


  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

protected:
  void staminaDash(WorldState& ws);
  double goalVal(const TVector& a, const TVector& b);
  
  ParameterView* parameterView;
  WorldStateView* worldStateView;
  TVector destination;
  double distance;
  double lookAtBall;

  double evalDir(WorldState& ws);

  bool goalFound(const WorldState& world);

  MoveSpeed speed;

  /** Hide copy-constructor. */
  FFMoveToPosition(const FFMoveToPosition&);
  /** Hide assignment operator. */
  FFMoveToPosition& operator= (const FFMoveToPosition&);
};


RS_END_NAMESPACE

#endif // !FFMOVETOPOSITION_H
