/* -*- Mode: C++ -*-
 * $Id: GotoBallAvoidingObstacles.cc,v 1.4 2001/09/29 18:55:20 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GotoBallAvoidingObstacles.cc
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-10-26
 * Last modified by $Author: frehe $ $Date: 2001/09/29 18:55:20 $
 *
 *
 * HISTORY
 * $Log: GotoBallAvoidingObstacles.cc,v $
 * Revision 1.4  2001/09/29 18:55:20  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.3  2001/09/28 14:05:10  andbo928
 * Fixed a serious bug where generateplan would be called infinitly.
 *
 * Revision 1.2  2001/09/26 11:41:25  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.1  2000/10/26 15:46:37  frehe
 * Initial version of GotoBallAvoidingObstacles.
 *
 */

/**
 * @file GotoBallAvoidingObstacles.cc
 * Implementation file for the skill class GotoBallAvoidingObstacles.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2001/09/29 18:55:20 $
 */


#include "MoveToPositionAvoidingObstacles.h"
#include "GotoBallAvoidingObstacles.h"
#include "BallView.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "DashCommand.h"


RS_BEGIN_NAMESPACE


GotoBallAvoidingObstacles::GotoBallAvoidingObstacles()
  : MoveToPositionAvoidingObstacles(),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
} 


GotoBallAvoidingObstacles::~GotoBallAvoidingObstacles()
{
  viewManager->releaseView(ballView->getId());
}


Plan GotoBallAvoidingObstacles::generatePlan()
{
  setPosition(ballView->getBallPosition(NOW).getPoint());
  return MoveToPositionAvoidingObstacles::generatePlan();
}


FuzzyBool GotoBallAvoidingObstacles::persistent()
{
  return FuzzyBool(0.0);
}


FuzzyBool GotoBallAvoidingObstacles::applicable()
{
  if (ballView->getBallPosition(NOW).isKnown()) {
    MoveToPositionAvoidingObstacles::setPosition(ballView->getBallPosition(NOW).getPoint());
    return MoveToPositionAvoidingObstacles::applicable();
  }
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE





