
/* -*- Mode: C++ -*-
 * $Id: GotoPassablePosition.cc,v 1.11 2001/10/10 14:50:17 aip12 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GotoPassablePosition.cc
 * Unitname: Framework/skill
 * $Revision: 1.11 $
 * Created by: Peter Andersson and Jonas Carlsson 
 * Last modified by $Author: aip12 $ $Date: 2001/10/10 14:50:17 $
 *
 *
 * HISTORY
 * $Log: GotoPassablePosition.cc,v $
 * Revision 1.11  2001/10/10 14:50:17  aip12
 * SIFindBall added. /Peter
 *
 * Revision 1.10  2001/10/09 13:05:40  aip12
 * Added so that you can chose whether to be or not to be (that is the question) in the line of sight of the player with the ball.
 *
 * Revision 1.9  2001/10/08 19:13:19  aip12
 * Updated applicable. Now it checks if some teammate has the ball too.
 *
 * Revision 1.7  2001/10/02 17:34:11  aip12
 * Fixat lite funktionalitet, fungerar hyffsat i vissa situationer.
 *
 * Revision 1.6  2001/09/30 16:31:31  aip12
 * Updated skill, nicer code, still lots of debug info, compiles after changes with FuzzyBool
 *
 * Revision 1.5  2001/09/29 18:55:21  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.4  2001/09/26 11:41:25  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.3  2001/09/24 16:51:29  aip12
 * Generate plan is "complete" (NOT TESTED!!!). You can compile it.
 *
 * Revision 1.2  2001/09/18 15:21:10  aip12
 * Skill implemented but not tested.
 *
 * Revision 1.1  2001/09/02 10:43:16  petan117
 * Added skill skeleton for GotoPassablePosition /Peter o Jonas
 *
 */

/**
 * @file GotoPassablePosition.cc
 * This is the implemention file for the skill GotoPassablePosition.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.11 $
 * $Author: aip12 $
 * $Date: 2001/10/10 14:50:17 $
 */

#include "GotoPassablePosition.h"
#include "ViewManager.h"
#include "IsBallKnown.h"
#include "FuzzyBool.h"
#include "TAngle.h"
#include "BallState.h"
//#include "WorldState.h"
#include "WorldStateView.h"
#include "PlayerState.h"
#include "SawBall.h"
#include "FFMoveToPosition.h"

RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

GotoPassablePosition::GotoPassablePosition()
  : Skill(),
    worldStateView()
{
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  //in case that the agent should take in considaration the ball position
  WorldState worldState = worldStateView->getWorldState();
  BallState ballState = worldState.getBall();
  gotoPassablePoint = ballState.getPosition();
}

GotoPassablePosition::GotoPassablePosition(TVector point)
  : Skill(),
    worldStateView()
{
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  //set the point which the agent take in considaration when it calculates the new position
  gotoPassablePoint = point;
}

// --- DESTRUCTOR ---

GotoPassablePosition::~GotoPassablePosition()
{
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}
  
// --- METHODS ---

//Procedure to set if the agent must be in line of site of the teammate who has the ball.
void GotoPassablePosition::setPlayerView(bool InPlayerView)
{
  inPlayerView = InPlayerView;
}

Plan GotoPassablePosition::generatePlan()
{
  WorldState worldState = worldStateView->getWorldState();
  BallState ballState = worldState.getBall();
  AgentState agentState = worldState.getAgent();

  //Variables
  TVector meToBallVector =  gotoPassablePoint - agentState.getPosition();
  TAngle rotateAngle = 88.0 - meToBallVector.getLength() / 5.0;
  //Calculate the "virtual" vector to se in which direction the agent should go
  TVector posVector1 = (gotoPassablePoint - agentState.getPosition()).rotate(rotateAngle);
  TVector posVector2 = (gotoPassablePoint - agentState.getPosition()).rotate(-rotateAngle);
  TVector newPosition1;
  TVector newPosition2;
  Step step;
  double coef1 = 0.0;
  double coef2 = 0.0;
  
  //Always look at the player we are making us passable for
  agentState.doTurnNeckTowards(gotoPassablePoint);
  
  //do the agent have to take in considaration the view of another player?
  if (inPlayerView)
    {
      //this for-loop is to calculate in which direction the agent should go
      //it for detecting a local minimun in the view 
      for (double i = 1.0; i < 12.0 ; i += 2.0)
	{
	  //Calculate the "virtual" vector to se in which direction the agent should go
	  newPosition1 = posVector1.setLength(meToBallVector.getLength() / i) + agentState.getPosition();
	  //Calculate how good is to go to that position
	  coef1 += getCoefSum(worldState, newPosition1, inPlayerView);
	  //Calculate the "virtual" vector to se in which direction the agent should go
	  newPosition2 = posVector2.setLength(meToBallVector.getLength() / i) + agentState.getPosition();
	  //Calculate how good is to go to that position
	  coef2 += getCoefSum(worldState, newPosition2, inPlayerView);
	}
    }
  else {
    newPosition1 = posVector1.setLength(meToBallVector.getLength() / 11) + agentState.getPosition();
    coef1 += getCoefSum(worldState, newPosition1, inPlayerView);
    newPosition2 = posVector2.setLength(meToBallVector.getLength() / 11) + agentState.getPosition();
    coef2 += getCoefSum(worldState, newPosition2, inPlayerView);
  }
  //which direction is the best
  if (coef1 < coef2){
    //run to newPosition1!
    step = doStep(posVector1, agentState);
  }
  else {
    //run to newPosition2!
    step = doStep(posVector2, agentState);
  }
  return Plan(step);
}

FuzzyBool GotoPassablePosition::persistent()
{
  return FuzzyBool(0.0);//FIXME
}

FuzzyBool GotoPassablePosition::applicable()
{
  WorldState worldState = worldStateView->getWorldState();
  FuzzyBool returnValue(FuzzyBool(0.0));
  TVector ballPosition = worldStateView->getWorldState().getBall().getPosition();

  //get all the known position of the oponents
  std::vector<PlayerState> opponents = worldStateView->getWorldState().getOpponents();

  // Encapsulate this instead of returning it directly since we want to add
  // pre-requisites later on...
  if (ballPosition.getC() >= 0.5 && opponents.size() >= 1){
    returnValue = FuzzyBool(1.0);
  }
  //A teammate have to have the ball
  if (hasTeammateBall(worldState) == NULL){
    returnValue = FuzzyBool(0.0);
  }
  
  return returnValue;
}

FuzzyBool GotoPassablePosition::succeed()
{
  return FuzzyBool(0.5);//FIXME
}

FuzzyBool GotoPassablePosition::failed()
{
  return FuzzyBool(0.5);//FIXME
}

Float GotoPassablePosition::dotProduct(TVector x, TVector y)
{
  return x.getX() * y.getX() + x.getY() * y.getY();
}

double GotoPassablePosition::abs(double a)
{
  return a >= 0 ? a : -a;
}

double GotoPassablePosition::getCoefSum(WorldState &worldState, TVector agentPosition, bool inPlayerView)
{
  BallState ballState = worldState.getBall();
  //AgentState agentState = worldState.getAgent();
  
  //get all the known position of the oponents
  std::vector<PlayerState> opponents = worldState.getOpponents();
  TVector meToBallVector;
  //Is it the ball position or a fixed position we are taking in consideration?
  if (gotoPassablePoint == ballState.getPosition()) {
    meToBallVector =  (ballState.getPosition() + ballState.getSpeed()) - agentPosition;
  } else {
    meToBallVector =  gotoPassablePoint - agentPosition;
  }
  double coef = 0;
  
  PlayerState teammateWithBall;
  //Get the teammate wit the ball
  teammateWithBall = *hasTeammateBall(worldState);
  TAngle teammateWithBallToAgentVectorAngle = (agentPosition - teammateWithBall.getPosition()).getAngle();

  //Get the sum that determins how good position we are in
  for (unsigned int i = 0;i<opponents.size();i++) {
    coef += (meToBallVector.getLength()
	     / getVectorsLength(opponents[i].getPosition() + opponents[i].getSpeed(), 
				gotoPassablePoint,
				agentPosition));
  }
  coef = coef / (double)opponents.size();  
  if (inPlayerView){
    coef += abs(teammateWithBall.getNeckDirection() - teammateWithBallToAgentVectorAngle) < 45 ? 0.0 : 1.0;
  }
  return coef;
}

//Procedure that determines what command to send to the server
Step GotoPassablePosition::doStep(TVector posVector, AgentState &agentState)
{
  //Get the angle between our present direction and our optimal direction
  TAngle angleDiff = posVector.getAngle() - agentState.getDirectionVector().getAngle();
  //check if agent needs to rotate ... if not run forward
  if (abs(angleDiff) < 10.0){
    agentState.doDash(80);
    return agentState.getStep();
  }
  //turn around to optimal direction
  else{
    agentState.doActualTurn(angleDiff);
    return agentState.getStep();
  }
  
}

//Procedure that finds out witch player has the ball and returns it
PlayerState *GotoPassablePosition::hasTeammateBall (WorldState &worldState)
{
  BallState ballState = worldState.getBall();
  
  std::vector<PlayerState> teammates = worldState.getTeammates();
  std::vector<PlayerState> opponents = worldState.getOpponents();
  
  double teammateLowestDistance = 120.0;
  double opponentLowestDistance = 120.0;
  int closestPlayerToBall = 0;
  //get the closest teammate to the ball
  for (unsigned int i = 0; i < teammates.size(); i++) {
    if (((teammates[i].getPosition() - ballState.getPosition()).getLength()) < 
	teammateLowestDistance) {
      teammateLowestDistance = ((teammates[i].getPosition() - ballState.getPosition()).getLength());
      closestPlayerToBall = i;
    }
  }
  //get the closest opponent to the ball
  for (unsigned int i = 0; i < opponents.size(); i++) {
    if (((opponents[i].getPosition() - ballState.getPosition()).getLength()) < 
	opponentLowestDistance) {
      opponentLowestDistance = ((opponents[i].getPosition() - ballState.getPosition()).getLength());
    }
  }
  //do the agent have the ball?
  if (teammateLowestDistance > (worldState.getAgent().getPosition() - ballState.getPosition()).getLength()){
    return NULL;
  } else   if (teammateLowestDistance < opponentLowestDistance){
    return &teammates[closestPlayerToBall];
  } else {
    return NULL;
  }
}

//Gets the lenght from the "passable point" to the opponent plus opponent to the agent
double GotoPassablePosition::getVectorsLength(TVector v1, TVector v2, TVector v3)
{
  return (v1 - v2).getLength() + (v1 - v3).getLength();
}

RS_END_NAMESPACE
