/* -*- Mode: C++ -*-
 * $Id: MakePassable.h,v 1.2 2001/10/11 09:15:35 aip3 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MakePassable.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Daniel Garpe, Karl Heijdenberg, Anders Lindgren 2001-10-08
 * Last modified by $Author: aip3 $ $Date: 2001/10/11 09:15:35 $
 *
 *
 * HISTORY
 * $Log: MakePassable.h,v $
 * Revision 1.2  2001/10/11 09:15:35  aip3
 * Changed pre- and postconditions in the MakePassable skill.
 *
 * Revision 1.1  2001/10/08 13:31:40  aip3
 * Added the MakePassable skill
 *
 */

/**
 * @file CutOffPassAngle.h
 * Include file for the skill class CutOffPassAngle.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip3 $
 * $Date: 2001/10/11 09:15:35 $
 */

#ifndef MAKEPASSABLE_H
#define MAKEPASSABLE_H

#include "Skill.h"
#include "MoveToPositionFacingDirectionAvoidingObstacles.h"

RS_BEGIN_NAMESPACE

class ParameterView;
class MarkerView;
class AgentView;
class BallView;
class GameView;
class PlayerView;

/**
 * @class MakePassable MakePassable.h
 *        This skill takes the player to a position where he's passable.  
 *
 * @pre There should be atleast one opponent closer to the teammate having the ball,  
 *      than the agent. The agent should also be within a 90 degree angle relative 
 *      the passer and the opponent.
 *
 * @post The player supposed to be passable has normally moved away from the opponent
 *       most likely to block the pass, in a direction bringing him closer to the passer.
 *       After the move the agent is facing the passer.
 *       If the new position is offside then the agent does not move.   
 *
 * @todo Check another oppenent, which possibly could block the pass and estimate best 
 *       position for passable player, relative both opponents. 
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip3 $
 * $Date: 2001/10/11 09:15:35 $
 */

class MakePassable : public Skill {
public:
  MakePassable();
  virtual ~MakePassable();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };
  
protected:
  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  GameView* gameView;
  PlayerView* playerView;
};

RS_END_NAMESPACE

#endif // !MAKEPASSABLE_H

