/* -*- Mode: C++ -*-
 * $Id: MarkPlayerFSAL.h,v 1.3 2001/10/09 18:50:09 aip4 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MarkPlayerFSAL.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Anders Lindahl 2001-09-29
 * Last modified by $Author: aip4 $ $Date: 2001/10/09 18:50:09 $
 *
 *
 * HISTORY
 * $Log: MarkPlayerFSAL.h,v $
 * Revision 1.3  2001/10/09 18:50:09  aip4
 * Added documentation
 *
 * Revision 1.2  2001/10/09 18:16:03  aip4
 * Seem to work as planned now
 *
 * Revision 1.1  2001/09/29 12:31:40  aip4
 * Added MarkPlayerFSAL skill.
 *
 */

/**
 * @file MarkPlayerFSAL.h
 * Include file for the skill class MarkPlayerFSAL.
 * <!--
 * @pre Ball and at least one opponent visible
 * @post Agent is close to one opponent, in line with the ball.
 * @invariant ?!?!
 *
 * @todo Finetune.
 * -->
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: aip4 $
 * $Date: 2001/10/09 18:50:09 $
 */


#ifndef MARKPLAYERFSAL_H
#define MARKPLAYERFSAL_H

//#include "SkillWithArguments.h"
#include "Skill.h"
#include "PlayerObject.h"
#include "WorldStateView.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "BallView.h"
#include "view_constants.h"
#include "DashCommand.h"
#include "TurnCommand.h"
#include "ServerCommand.h"
#include "Time.h"

RS_BEGIN_NAMESPACE

class WorldStateView;
class ParameterView;
class AgentView;
class BallView;
class PlayerView;

class MarkPlayerFSAL : public Skill {
 public:
    /** Default constructor. Here you can initialize the skill. */
  MarkPlayerFSAL();

  /** Destructor should clean up after the skill. */
  virtual ~MarkPlayerFSAL();

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();



  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();
  
  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
    virtual FuzzyBool succeed() { return FuzzyBool(0.5); }; 

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
   virtual FuzzyBool failed() { return FuzzyBool(0.5); }; 

protected:
  /** Returns FuzzyBool(1.0) if a player is marked */
  virtual FuzzyBool playerMarked(PlayerObject player);
    
  /** Returns a pointer to the closest unmarked opponent, if any. */
  virtual PlayerObject* getClosestUnmarkedOpponent();
  
  WorldStateView* worldStateView;
  ParameterView* parameterView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
  
  PlayerObject* markedOpponent;
  PointUCO ballLastPosition;

  Int opponentTime;
  Int ballTime;

  int ballMemory;
  int opponentMemory;

  std::vector<PlayerObject> teammates;
  std::vector<PlayerObject> opponents;
  
};

RS_END_NAMESPACE

#endif // !MARKPLAYERFSAL_H











