/* -*- C++ -*- */
   
#ifndef PASS_H
#define PASS_H


#include "Skill.h"
#include "SkillWithArguments.h"
#include "PlayerObject.h"
#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


class ParameterView;
class AgentView;
class BallView;
class TerritoryControlView;
class PlayerView;


/**
 * @class Pass Pass.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.10 $
 * $Author: peljo105 $
 * $Date: 2001/09/05 14:57:59 $
 */

class Pass : public SkillWithArguments {
public:
  /** Default constructor. Here you can initialize the skill. */
  Pass();

  /** Destructor should clean up after the skill. */
  virtual ~Pass();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();
  
  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
  /** Used to set the skill's arguments in a generic way.
   This method must be overridden by subclasses.
   It should transform the generic vector of floats to
   the actual arguments.
   It will throw TooFewArgumentsException if not enough arguments
   are given. */
  virtual void setArguments(const std::vector<Float>& arguments)
    throw(RS::TooFewArgumentsException);


  /** This method is used to first set the arguments of the skill
    and then generate a plan for how the goal of the skill should
    be obtained by the agent. It uses the methods setArguments and
    generatePlan(), so it should not be overriden unless it should
    made unavailable. */
  virtual Plan generatePlan(const std::vector<Float>& arguments);
  virtual FuzzyBool applicable(const std::vector<Float>& arguments);
  virtual Vector calculatePassVector();

protected:

  PlayerObject findClosestTeammate();
  ParameterView* parameterView;
  //  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
  TerritoryControlView* territoryControlView;
  std::vector<PlayerObject> Pass::seenVictims();
  std::vector<int> victims;

  
  /** Hide copy-constructor. */
  Pass(const Pass&);
  /** Hide assignment operator. */
  Pass& operator= (const Pass&);
};


RS_END_NAMESPACE

#endif // !PASS_H
