/* -*- Mode: C++ -*-
 * $Id: PassForGoalChance.h,v 1.6 2001/10/10 05:58:58 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassForGoalChance.h
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Pelle Johansson, Mikael Lindstrm, Per-Ola Kristensson
 * Last modified by $Author: aip10 $ $Date: 2001/10/10 05:58:58 $
 *
 *
 * HISTORY
 *
 * $Log: PassForGoalChance.h,v $
 * Revision 1.6  2001/10/10 05:58:58  aip10
 * Fixed @todo
 *
 * Revision 1.5  2001/10/10 05:14:15  aip10
 * Smarter CanScore
 *
 * Revision 1.4  2001/10/03 08:55:52  aip10
 * Switched to WorldStateView and added offside check.
 *
 * Revision 1.3  2001/09/28 11:45:49  aip10
 * Updated PassForGoalChance
 *
 * Revision 1.2  2001/09/05 14:58:27  peljo105
 * Implemented calculatePassVector() and CanPass()
 *
 * Revision 1.1  2001/09/05 11:41:44  peljo105
 * Frsta version
 *
 *
 */

/**
 * @file PassForGoalChance.h
 * 
 *
 * @par Unit:
 *      Framework/skill
 *
 * $Revision: 1.6 $
 * $Author: aip10 $
 * $Date: 2001/10/10 05:58:58 $
 */


#ifndef PassForGoalChance_H
#define PassForGoalChance_H


// Include the header files which definitions you use.
#include "Pass.h"
#include "WorldStateView.h"


RS_BEGIN_NAMESPACE

class MarkerView;

/**
 * @class PassForGoalChance PassForGoalChance.h
 * This skill passes the ball to a player with a chance to score, provided
 * we don't.
 *
 * @pre The preconditions for the skill are:
 *      - We have the ball.
 *      - There is a player that has a chance to score.
 *      - We can pass that player.
 *      - That player is not offside.
 *
 * @post The inteded postconditions for this skill are:
 *      - The player with best chance to score has the ball.
 *
 * @todo There are currently nothing left to do.
 *
 * @par Unit:
 *      Framework/skill
 */
class PassForGoalChance : public Pass {
public:
  /** Default constructor. Here you can initialize the skill. */
  PassForGoalChance();

  /** Destructor should clean up after the skill. */
  virtual ~PassForGoalChance();
  
  virtual FuzzyBool applicable();
  virtual Vector calculatePassVector();
  virtual FuzzyBool CanScore(PlayerState &player,std::vector<PlayerState> &foes);

private:
  /** Description of this attribute. */
  // TheType attributeName;

  
  /** Hide copy-constructor. */
  PassForGoalChance(const PassForGoalChance&);
  /** Hide assignment operator. */
  PassForGoalChance& operator= (const PassForGoalChance&);

protected:
  
  MarkerView* markerView;
  WorldStateView *worldView;
};


RS_END_NAMESPACE


#endif // PassForGoalChance_H
