/* -*- Mode: C++ -*-
 * $Id: ReceivePass.cc,v 1.4 2001/09/29 18:55:22 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ReceivePass.cc
 * Unitname: Framework/skill
 *	
 * HISTORY
 *
 * $Log: ReceivePass.cc,v $
 * Revision 1.4  2001/09/29 18:55:22  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.3  2001/09/28 12:59:44  aip9
 * *** empty log message ***
 *
 * Revision 1.2  2001/09/26 11:41:27  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.1  2001/09/14 13:12:26  aip9
 * Added the ReceivePass skill.
 *
 */

/**
 * @file ReceivePass.cc
 * Implementation file for the skill class ReceivePass.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2001/09/29 18:55:22 $
 */

#include "Skill.h"
#include "ReceivePass.h"

#include "BallKickedToMe.h"

#include "BallView.h"
#include "ParameterView.h"

#include "view_constants.h"
#include "math_utils.h"

#include "TurnCommand.h"
#include "DashCommand.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ReceivePass::ReceivePass()
  :Skill(),
   ballView(NULL),
   parameterView(NULL),
   agentView(NULL)
  // This is an initializer list
  // which means that all the objects inside the
  // class can be initialized after the colon.
  // You initialize the variable myVar with
  // myVar(constructor arguments).
  // You should also initialize the base classes, but
  // only the classes that you directly inherit from.
  // If you do not initialize a variable its default
  // constructor will be called.
{
  getView(worldView, WORLD_STATE_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}


// --- DESTRUCTOR ---

ReceivePass::~ReceivePass()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}


// --- METHODS ---

Plan ReceivePass::generatePlan()
{
  // Return an empty plan.
  return Plan();
}


FuzzyBool ReceivePass::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool ReceivePass::applicable()
{
  BallKickedToMe bk(10.0,4);
  return (bk.evaluate());
  
}

FuzzyBool ReceivePass::succeed()
{
  // Always succedes.
  return FuzzyBool(1.0);
}

FuzzyBool ReceivePass::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}



RS_END_NAMESPACE

