/* -*- Mode: C++ -*-
 * $Id: ReceivePass.h,v 1.2 2001/09/28 14:09:59 aip9 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ReceivePass.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Karel Salasek & Felix Maurer 2001-09-14
 * Last modified by $Author: aip9 $ $Date: 2001/09/28 14:09:59 $
 *
 *
 * HISTORY
 *
 * $Log: ReceivePass.h,v $
 * Revision 1.2  2001/09/28 14:09:59  aip9
 * ???
 *
 * Revision 1.1  2001/09/14 13:12:26  aip9
 * Added the ReceivePass skill.
 *
 */

/**
 * @file ReceivePass.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip9 $
 * $Date: 2001/09/28 14:09:59 $
 */


#ifndef RECEIVEPASS_H
#define RECEIVEPASS_H


#include "Skill.h"
#include "framework_types.h"
#include "WorldStateView.h"

RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;

/**
 * @class ReceivePass ReceivePass.h
 * Receive a ball kicked in our direction.
 * The rest is a longer description of the skill class ReceivePass.
 * Use it to describe the skill!
 *
 * @pre 
 * @post The intended postconditions for the skill are...
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip9 $
 * $Date: 2001/09/28 14:09:59 $
 */
class ReceivePass : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  ReceivePass();

  /** Destructor should clean up after the skill. */
  virtual ~ReceivePass();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

private: 

  BallView* ballView;
  ParameterView* parameterView;
  AgentView* agentView;
  
  WorldStateView* worldView;

  /** Hide copy-constructor. */
  ReceivePass(const ReceivePass&);
  /** Hide assignment operator. */
  ReceivePass& operator= (const ReceivePass&);
};


RS_END_NAMESPACE

#endif // !RECEIVEPASS_H



















