#include "SIFindBall.h"

RS_BEGIN_NAMESPACE

static bool keepLooking = false;

SIFindBall::SIFindBall()
  : Skill(),
    worldStateView(NULL)
{
  getView(worldStateView, WORLD_STATE_VIEW_ID);
}


SIFindBall::~SIFindBall()
{
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}


Plan SIFindBall::generatePlan()
{
  WorldState worldState = worldStateView->getWorldState();
  BallState ballState = worldState.getBall();
  AgentState agentState = worldState.getAgent();

  if (keepLooking && agentState.doTurn(45.0)) {
    //Do nothing, get the step below
  } else {
    
    // Try to calculate the ball's position and turn head or body there
    int time = worldState.getGameTime() - ballState.getGameTime();
    TVector newPosition = ballState.getPosition() + ballState.getSpeed() * time;
    
    if (agentState.doTurnNeckTowards(newPosition)) {
      //Do nothing, get step below
    }
    else if (agentState.doTurnTowards(newPosition)) {
    }
    keepLooking = true;
  }
  return Plan(agentState.getStep());
}


FuzzyBool SIFindBall::persistent()
{
  return FuzzyBool(0.0);
}


FuzzyBool SIFindBall::applicable()
{
  FuzzyBool applicable = FuzzyBool(1-worldStateView->getWorldState().getBall().getC());
  if (applicable.isFalse()) keepLooking = false;
  return applicable;
}

FuzzyBool SIFindBall::succeed()
{
  return FuzzyBool(1.0);
}

FuzzyBool SIFindBall::failed()
{
  return FuzzyBool(0.0);
}

RS_END_NAMESPACE
