/* -*- Mode: C++ -*-
 * $Id: TestTeamDecision.h,v 1.3 2001/09/01 15:06:52 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TestTeamDecision.h
 * Unitname: Framework/test/TestTeam
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-16
 * Last modified by $Author: frehe $ $Date: 2001/09/01 15:06:52 $
 *
 *
 * HISTORY
 *
 * $Log: TestTeamDecision.h,v $
 * Revision 1.3  2001/09/01 15:06:52  frehe
 * Updated the file to work with the current version of RoboSoc.
 *
 * Revision 1.2  2000/09/16 18:39:42  frehe
 * Fixed the log messages.
 *
 * Revision 1.1  2000/09/16 18:37:09  frehe
 * Add a team that can be used as the basis for test teams.
 *
 */

/**
 * @file TeamDecision.h
 * Include file for the class TeamDecision.
 *
 * @par Unit:
 *      Framework/test/TestTeam
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/09/01 15:06:52 $
 */


#ifndef TEAMDECISION_H
#define TEAMDECISION_H


#include "basic_system_compiler_directives.h"
#include "Decision.h"

//#define RS_USE_COACH_DEBUGGING
#define RS_USE_DISPLAY

#ifdef RS_USE_DISPLAY
#  include "Display.h"
#endif

//#define RS_USE_OWN_LOGFILE


#ifdef RS_USE_COACH_DEBUGGING
#  include "CoachView.h"
#endif

#ifdef RS_USE_OWN_LOGFILE
#  include <fstream>
#endif


RS_BEGIN_NAMESPACE


class ViewManager;
class ParameterView;
class GameView;
class CommandView;
class BallView;
class AgentView;
class PlayerView;
class MarkerView;


/**
 * @class TeamDecision TeamDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Framework/test/TestTeam
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/09/01 15:06:52 $
 */
class TeamDecision : public Decision {
public:
  TeamDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~TeamDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  //virtual bool init(ActuatorInterface* const actuator_interface);
  virtual bool decide();

  // Event handlers
  virtual void onActuatorSensorData();
  //virtual void onAuralSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  ViewManager* const viewManager;
  ParameterView* parameterView;
  GameView* gameView;
  CommandView* commandView;
  MarkerView* markerView;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
  bool serverAlive;

#ifdef RS_USE_DISPLAY
  Display theDisplay;
#endif

#ifdef RS_USE_COACH_DEBUGGING
  CoachView* coachView;
#endif

#ifdef RS_USE_OWN_LOGFILE
  std::ofstream logFile;
#endif
};


RS_END_NAMESPACE


#endif // !TEAMDECISION_H
