/*
 * $Id: PlayerState.cc,v 1.1 2001/09/18 20:41:48 andbo928 Exp $
 * 
 * HISTORY 
 *
 * $Log: PlayerState.cc,v $
 * Revision 1.1  2001/09/18 20:41:48  andbo928
 * Imported some sources from FuzzyFoo.
 *
 * Revision 1.51  2001/07/28 15:42:57  mikbr212
 * Added new doKickToSpeedAvoidCollisions
 *
 * Revision 1.50  2001/07/28 15:12:04  mikbr212
 * fixed doKickFoo
 *
 * Revision 1.49  2001/07/28 14:11:01  andbo928
 * Fixed some minor bugs in doKick.
 *
 * Revision 1.48  2001/07/28 14:08:47  andbo928
 * Fixed some minor bugs in doDash.
 *
 * Revision 1.47  2001/07/28 09:22:46  mikbr212
 * Cosmetic changes
 *
 * Revision 1.46  2001/07/28 07:18:05  mikbr212
 * Removed warnings!!
 *
 * Revision 1.45  2001/07/28 06:51:12  mikbr212
 * Removed compile errors
 *
 * Revision 1.44  2001/07/27 15:52:05  andbo928
 * Fixed a ton of warnings. NR intercept thingy added (but not used).
 *
 * Revision 1.43  2001/07/27 10:31:37  mikbr212
 * Fixed assignment operator in AgentState
 *
 * Revision 1.42  2001/07/26 12:40:58  mikbr212
 * removed PlayerState::getTeam()
 *
 * Revision 1.41  2001/07/16 15:42:08  andbo928
 * Fixed ball confidence reduction in icept calculation.
 *
 * Revision 1.40  2001/07/09 16:53:40  andbo928
 * Added extra heuristics to getInterceptTime.
 *
 * Revision 1.39  2001/07/06 10:00:38  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.38  2001/07/05 17:11:53  andbo928
 * Removed get/setTeam, added Agent=Player assignemnt opertaor.
 *
 * Revision 1.37  2001/07/04 12:09:38  andbo928
 * Removed some unused attributes.
 *
 * Revision 1.36  2001/07/02 14:45:33  andbo928
 * Added canIntercept method.
 *
 * Revision 1.35  2001/07/01 14:15:05  sigge
 * Added Mickes code (Statistics view)
 *
 * Revision 1.34  2001/07/01 12:38:29  andbo928
 * Added new limitations imposed by the new server in simulation. Added doChangeViewFoo methods.
 *
 * Revision 1.33  2001/06/19 15:22:47  mikbr212
 * Now uses the team-value in the player
 *
 * Revision 1.32  2001/06/19 14:02:09  mikbr212
 * Added set and get methods for intention and team.
 *
 * Revision 1.31  2001/06/05 20:33:53  mikbr212
 * Fixed doxygen-stubs for all classes
 *
 */

/**
 * @file PlayerState.cc
 * Implementation file for the classes PlayerState and AgentState.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: andbo928 $
 * $Date: 2001/09/18 20:41:48 $
 */

#include "PlayerState.h"

#include "BallState.h"

#include "TLine.h"

#include "DashCommand.h"
#include "TurnCommand.h"
#include "KickCommand.h"
#include "TurnNeckCommand.h"
#include "ChangeViewCommand.h"

RS_BEGIN_NAMESPACE

//ParameterView* PlayerState::parameterView;

PlayerState::PlayerState() :
  speed(TVector(0, 0, 0)),
  direction(0),
  neckDirection(0),
  position(TVector(0, 0, 0)),
  uniformNumber(-1),
  gameTime(0),
  intention(NULL),
  nextCommand(cmdNull),
  nextCommand2(cmdNull),
  nextCommand3(cmdNull),
  lastCommand(cmdNull),
  turnAngle(0),
  dashPower(0),
  kickVector(TVector(0,0)),
  turnNeckAngle(0),
  parameterView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
}

PlayerState::~PlayerState()
{
  ViewManager::instance()->releaseView(PARAMETER_VIEW_ID);  
}

/** Merges this player with the other. The position and speed
 * with the highest confidence is choosen. The rest of
 * the values is taken from the other agent (if known).
 */
void PlayerState::merge(const PlayerState& other) {
  speed = speed.confSelect(other.speed);
  position = position.confSelect(other.position);
  if (other.uniformNumber != -1)
    uniformNumber = other.getUniformNumber();
}

bool PlayerState::operator==(const PlayerState& other) const
{
  return
    speed == other.speed
    && position == other.position
    && direction == other.direction
    && neckDirection == other.neckDirection
    && uniformNumber == other.uniformNumber
    && gameTime == other.gameTime;
}

bool PlayerState::overlaps(const BallState& other) const
{
  TVector delta = other.getPosition() - position;
  if(delta.getLength() < 1.05 * (parameterView->SP_player_size
				 + parameterView->SP_ball_size))
  {
//      cout << other.getPosition() << ", " << position << endl;
//      cout << delta.getLength() << endl;
    return true;
  }
  return false;
}

bool PlayerState::overlaps(const PlayerState& other) const
{
  TVector delta = other.getPosition() - position;
  if(delta.getLength() < 2.1 * parameterView->SP_player_size)
    {
      return true;
    }
  return false;
}

bool PlayerState::doKick(const TVector& v, BallState& ball)
{
  bool validKick = true;
  nextCommand = cmdKick;
  kickVector = v;

  if(kickVector.getLength() > parameterView->SP_maxpower) {
    validKick = false;
    kickVector.setLength(parameterView->SP_maxpower);
  }

  // Add acceleration to ball
  double kickPowerRate = getKickPowerRate(ball);
  if(kickPowerRate == 0) {
    validKick = false;
  } else {
    TVector ballAcc(kickVector);
    ballAcc.rotate(getDirection());
    ballAcc *= kickPowerRate;
    ball.setAcceleration(ball.getAcceleration() + ballAcc);
    
    ball.step();
    if(overlaps(ball))
      validKick = false;
  }

  return validKick;
}

bool PlayerState::isBallKickable(const BallState& ball) const {
  float ballDist = (ball.getPosition()-getPosition()).getLength();
  return ballDist < parameterView->SP_kickable_area;
}

inline float PlayerState::nr_f(float t, float A, float B,
			       float C, float D, float E) const
{
  float d = parameterView->SP_ball_decay;
  float dt = pow(d, t);
  return A*dt*dt + B*dt + C*t*t + D*t + E;
}

inline float PlayerState::nr_fdif(float t, float A, float B, 
				  float C, float D, float ) const
{
  float d = parameterView->SP_ball_decay;
  float ld = log(d);
  float dt = pow(d, t);
  return 2*A*ld*dt*dt + B*ld*dt + 2*C*t + D;
}

float PlayerState::tToS(float t) const
{
  return pow(parameterView->SP_ball_decay, t) - 1;
}

float PlayerState::sToT(float s) const
{
  return log(s+1) / log(parameterView->SP_ball_decay);
}

int PlayerState::getInterceptTimeNR(const BallState& ball, int ) const
{
  // Find approx solutions.
  // Second degree solution to the intercept dilemma.

  TVector u = ball.getPosition() - getPosition();
  TVector v0 = ball.getSpeed();
  TVector p0 = getPosition();
  TVector b0 = ball.getPosition();
  float ka = parameterView->SP_kickable_area;
  float vp = parameterView->SP_player_speed_max;
  float d = parameterView->SP_ball_decay;

  float f = v0*v0 - vp*vp;
  float p = (2*u*v0 - 2*ka*vp) / f;
  float q = (u*u - ka*ka) / f;

  float sq = p*p/4 - q;

  float tGuess = 0.0;
  // Guess initally at the end position using crude heuristics.
  int guessNo = 1;
  tGuess = 
    ((b0 + v0 * -1/(d-1) - p0).getLength() - ka) / vp; 

  if(sq > 0)
  {
    sq = sqrt(sq);
    float t1 = -p/2 - sq;
    float t2 = -p/2 + sq; 
    if(t2 > 0 && t2 < tGuess)
    {
      // Solution 2 is a better guess.
      guessNo = 2;
      tGuess = t2;
    }

    if(t1 > 0 && t1 < tGuess)
    {
      // Solution 1 is a better guess.
      guessNo = 3;
      tGuess = t1;
    }
  }

  // Newton-Raphson it. (On a d^t-1 scale. ;-D)
  float A = v0*v0 / ((d-1)*(d-1));
  float B = -2*A + 2*u*v0 / (d-1);
  float C = -vp*vp;
  float D = -2*ka*abs(vp);
  float E = u*u - A - B - ka*ka;

  float t = tGuess;
  float lastT;
  float error = 1;
  int benchMark = 0;
  while(abs(error) > 0.001 && benchMark < 20)
  {
    lastT = t;
    t = t - nr_f(t, A, B, C, D, E)/nr_fdif(t, A, B, C, D, E);
    error = lastT - t;
    benchMark++;
  }

//    int delta = (int) (tStep - t);
//    if(abs(delta) > 1 && tStep != -1)
//    {
//      cout << "NR/Stepping mismatch time: " << tStep << ", NR: " << t << endl;
//      cout << "guessNo = " << guessNo << ";";
//      cout << "tGuess = " << tGuess << ";";
//      cout << "v0 = " << ball.getSpeed().getX() << "+" 
//  	 << ball.getSpeed().getY() << "*i;";
//      cout << "b0 = " << ball.getPosition().getX() << "+" 
//  	 << ball.getPosition().getY() << "*i;";
//      cout << "p0 = " << getPosition().getX() << "+" 
//  	 << getPosition().getY() << "*i;";
//      cout << endl;
//    }

//    if(isnan(t))
//    {
//      cout << "Got NaN:" << endl;
//      cout << "guessNo = " << guessNo << ";";
//      cout << "tGuess = " << tGuess << ";";
//      cout << "v0 = " << ball.getSpeed().getX() << "+" 
//  	 << ball.getSpeed().getY() << "*i;";
//      cout << "b0 = " << ball.getPosition().getX() << "+" 
//  	 << ball.getPosition().getY() << "*i;";
//      cout << "p0 = " << getPosition().getX() << "+" 
//  	 << getPosition().getY() << "*i;";
//      cout << endl;

//      //    cout << "Ouch. Nan in NR." << endl;
//    }
//    cout << "NR-solution(" << benchMark << "): " << t << endl;
  return (int)ceil(t);
}

int PlayerState::getInterceptTime(const BallState& ball) const 
{
  BallState bs = ball;
  int time = -1;
#define MAX_CHECK_STEPS 20
  for(unsigned int i = 0; i < MAX_CHECK_STEPS; i++)
  {
    TVector delta = 
      bs.getPosition()
      - getPosition();
    if(delta.getLength() < (i * parameterView->SP_player_speed_max
			    + parameterView->SP_kickable_area))
    {
      time = i;
      break;
    }
    bs.setC(1.0);
    bs.step();
  }

//    int nr = getInterceptTimeNR(ball, time);

  if(time == -1)
  {
    // Very coarse heuristics.
    TVector delta = ball.getPosition() - getPosition();
    time = (int)(1.1 * delta.getLength() / parameterView->SP_player_speed_max);
  }

  return time;
}

bool PlayerState::canIntercept(const BallState& ball) const
{
  // Heuristics...
  PlayerState clone = *this;
  int deltaTime = ball.getGameTime() - gameTime;

  // FIXME: This is heuristics, change this to an analytical expression.
  for(int i = 0; i < deltaTime; i++)
  {
    clone.doNull();
    clone.step();
  }

  TVector delta = ball.getPosition() - clone.getPosition();
  if(delta.getLength() < ((deltaTime - 1) * parameterView->SP_player_speed_max
			  + parameterView->SP_kickable_area))
  {
    return true;
  }
  return false;
}

double PlayerState::getKickPowerRate(const BallState& ball) const {
  TVector relBallPos(ball.getPosition() - getPosition());
  relBallPos.rotate(-getDirection());

  if(relBallPos.getLength() > parameterView->SP_kickable_area)
    return 0;

  return parameterView->SP_kick_power_rate 
    * (1.0 - 0.25 * abs(relBallPos.getAngle())
       / 180 - 0.25 * (relBallPos.getLength() 
		       - parameterView->SP_player_size 
		       - parameterView->SP_ball_size)
       / parameterView->SP_kickable_margin);
}

/** Does a kick to the given position. If it is possible then true
 * is returned. The agent will try to avoid collisions.
 */
bool PlayerState::doKickToPosition(const TVector& pos, BallState& ball) {

  double t;
  TLine kickLine(ball.getPosition(), pos);
  TVector distance(kickLine.getDistance(getPosition(), t));

  bool retVal = true;

  if(t >= 0
     && t <= 1.0
     && distance.getLength() < (parameterView->SP_player_size
				+ parameterView->SP_ball_size) * 1.05) {
    // can't kick straight line, adjust!
    distance = -distance;
    distance.setLength((parameterView->SP_player_size
			+ parameterView->SP_ball_size) * 1.05);
    kickLine.setVector(distance + (getPosition() - ball.getPosition()));
    retVal = false;
  }

  if(doKickToSpeed(kickLine.getVector(), ball))
    return retVal;

  return false;
}

/** Does a kick to (about) the given speed. If it is possible then true
 * is returned. The agent will try to avoid collisions, but
 * if there will be a collision then false is returned.
 * That is, only when the wanted speed is achieved without collisions
 * true will be returned.
 */
bool PlayerState::doKickToSpeedAvoidCollisions(const TVector& speed,
					       BallState& ball)
{
  return doKickToPosition(ball.getPosition() + speed, ball);
}


/** Does a kick to the given speed. If it is possible then true
 * is returned. The agent will NOT try to avoid collisions, but
 * if there will be a collision then false is returned.
 * That is, only when the wanted speed is achieved without collisions
 * true will be returned.
 */
bool PlayerState::doKickToSpeed(const TVector& speed, BallState& ball) 
{
  TVector ballAcc(0,0);
  double kickPowerRate = getKickPowerRate(ball); 
  if(kickPowerRate == 0)
    return false;

  double maxAcc = 100 * kickPowerRate;
  if(maxAcc > parameterView->SP_ball_accel_max)
    maxAcc = parameterView->SP_ball_accel_max;

  bool canKick = false;

  if((speed - ball.getSpeed()).getLength() > maxAcc) { 
    // Can't kick to desired speed, try to kick in right direction
    
    TLine wantedDir(TVector(0,0), speed);
    double t;
    TVector d = wantedDir.getDistance(ball.getSpeed(), t);
    
    if(abs(AngleDeg(ball.getSpeed().getAngle() 
		    -speed.getAngle()).getDeg()) >= 90) 
    {
      // Ball moving in the wrong direction (diff > 90)
      
      if(ball.getSpeed().getLength() >= maxAcc) {
	// ball moving to fast, try slow it down
	ballAcc = - ball.getSpeed();
	ballAcc.setLength(maxAcc);
      } else {
	ballAcc = -d;
	ballAcc += TVector::TPolarVector(sqrt(maxAcc*maxAcc - d*d),
					 speed.getAngle());
      }
    } else {
      // Ball moving in the right direction (diff < 90)
      
      if(d.getLength() >= maxAcc) {
	// Ball moving too fast, try to put it in almost the right direction
	ballAcc = -d;
	ballAcc.setLength(maxAcc);
      } else {
	// Possible to kick in right direction, but not with right speed
	ballAcc = -d;
	
	if(t > 1.0) // ball will move to fast (in the right dir)
	  ballAcc -= TVector::TPolarVector(sqrt(maxAcc*maxAcc - d*d),
					   speed.getAngle());
	else // ball will move to slow (in the right dir)
	  ballAcc += TVector::TPolarVector(sqrt(maxAcc*maxAcc - d*d),
					   speed.getAngle());
      }
    }
  } else {
    // Just kick!
    ballAcc = speed - ball.getSpeed();
    canKick = true;
  }

  //   Adjust length to power instead of acceleration
  TVector kVec = ballAcc / kickPowerRate;
  kVec.rotate(-direction);
  canKick = doKick(kVec, ball) && canKick;

  if(canKick) { // Check collision
    BallState newBall(ball);
    PlayerState newAgent(*this);
    newBall.step();
    newAgent.step();
    canKick = !(newAgent.overlaps(newBall));
  }
  return canKick;
}

bool PlayerState::doDash(float power)
{
  nextCommand = cmdDash;

  if(power > parameterView->SP_maxpower)
  {
    dashPower = parameterView->SP_maxpower;
    return false;
  }

  if(power < parameterView->SP_minpower)
  {
    dashPower = parameterView->SP_minpower;
    return false;
  }

  dashPower = power;
  return true;
}

bool PlayerState::doDashDistance(float dist)
{
  double distLeft = 
    ((getDirectionVector() * dist) - speed) * getDirectionVector();
  double powerNeed = distLeft / parameterView->SP_dash_power_rate;
  return doDash(powerNeed);
}

/** Dash to the given speed. Since this will only do a dash
 * (not a turn), the speed in the players direction will
 * only be affected. That is, if the resulting speed is divided
 * into two composants, one in the players direction and the other
 * orthogonal to the first, the amount of speed along the second
 * composant will we ignored.
 * @param speed the amount of speed wanted. Negative means backwards.
 * @return true if the desired speed was achieved.
 */
bool PlayerState::doDashToSpeed(float speed) {
  float neededSpeed = speed - getSpeed() * getDirectionVector();
  return doDash(neededSpeed / parameterView->SP_dash_power_rate);
}

bool PlayerState::doTurn(float angle)
{
  nextCommand = cmdTurn;
  turnAngle = angle;

  bool validTurn = true;
  if(turnAngle < -180.0)
  {
    validTurn = false;
    turnAngle = -180.0;
  }
  if(turnAngle > 180.0)
  {
    validTurn = false;
    turnAngle = 180.0;
  }

  return validTurn;
}

bool PlayerState::doActualTurn(float angle)
{
  double neededAngle = 
    angle * (1.0 + parameterView->SP_inertia_moment * speed.getLength());
  return doTurn(neededAngle);
}

bool PlayerState::doTurnTo(float dir)
{
  TAngle delta = dir - direction;
  return doActualTurn(delta);
}

bool PlayerState::doTurnTowards(const TVector& position)
{
  PlayerState nextPlr = *this;
  nextPlr.step();
  TVector turnVec = position - nextPlr.getPosition();
  return doTurnTo(turnVec.getAngle());
}

bool PlayerState::doTurnNeck(float angle)
{
  nextCommand2 = cmdTurnNeck;
  turnNeckAngle = angle;

  bool validTurnNeck = true;
  double nextNeck = turnNeckAngle + neckDirection;
  if(nextNeck > 90.0)
    {
      validTurnNeck = false;
      turnNeckAngle = 90 - neckDirection;
    }
  if(nextNeck < -90.0)
    {
      validTurnNeck = false;
      turnNeckAngle = -90 - neckDirection;
    }

  return validTurnNeck;
}

bool PlayerState::doTurnNeckTo(float dir)
{
  // Adjust for dashes too.
  TAngle delta = dir - direction - neckDirection;

  if(nextCommand == cmdTurn)
    {
      // Might be better to step an agentstate. hmm...
      // We won't thou...
      delta = delta - getActualTurn(turnAngle);
    }
  return doTurnNeck(delta);
}

bool PlayerState::doTurnNeckTowards(const TVector& position)
{
  PlayerState nextPlr = *this;
  nextPlr.step();

  TVector lookVec = position - nextPlr.getPosition();

  return doTurnNeckTo(lookVec.getAngle());
}

Step PlayerState::getStep()
{
  Step stp;

  if(nextCommand == cmdDash)  {
      DashCommand dash = DashCommand(dashPower);
      //      cout << "getStep => " << dash << endl;
      stp.addCommand(&dash);
  } else if(nextCommand == cmdTurn) {
      TurnCommand turn(turnAngle);
      //      cout << "getStep => " << turn << endl;
      stp.addCommand(&turn);
  } else if(nextCommand == cmdKick) {
      KickCommand kick(kickVector.getLength(),
		       kickVector.getAngle());
      //      cout << "getStep => " << kick << endl;
      stp.addCommand(&kick);
  } else {
      TurnCommand null(0);
      //      cout << "getStep => " << null << endl;
      stp.addCommand(&null);
  }

  if(nextCommand2 == cmdTurnNeck)
  {
    TurnNeckCommand turnNeck(turnNeckAngle);
    stp.addCommand(&turnNeck);
  }

  if(nextCommand3 == cmdChangeView)
  {
    ChangeViewCommand viewChangeCommand(VW_NORMAL, VQ_HIGH);
    switch(vQuality)
    {
    case vqLow:
      viewChangeCommand.setViewQuality(VQ_LOW);
      break;
    case vqHigh:
    default:
      viewChangeCommand.setViewQuality(VQ_HIGH);
      break;
    }

    switch(vWidth)
    {
    case vwNarrow:
      viewChangeCommand.setViewWidth(VW_NARROW);
      break;
    case vwWide:
      viewChangeCommand.setViewWidth(VW_WIDE);
      break;
    case vwNormal:
    default:
      viewChangeCommand.setViewWidth(VW_NARROW);
      break;
    }
    stp.addCommand(&viewChangeCommand);
  }

  return stp;
}

bool PlayerState::doNull()
{
  nextCommand = cmdNull;
  return true;
}

bool  PlayerState::doChangeViewWide()
{
  nextCommand3 = cmdChangeView;
  vWidth = vwWide;
  return true;
}

bool  PlayerState::doChangeViewNormal()
{
  nextCommand3 = cmdChangeView;
  vWidth = vwNormal;
  return true;
}

bool  PlayerState::doChangeViewNarrow()
{
  nextCommand3 = cmdChangeView;
  vWidth = vwNarrow;
  return true;
}

bool  PlayerState::doChangeViewHigh()
{
  nextCommand3 = cmdChangeView;
  vQuality = vqHigh;
  return true;
}

bool  PlayerState::doChangeViewLow()
{
  nextCommand3 = cmdChangeView;
  vQuality = vqLow;
  return true;
}

TVector PlayerState::getSpeed() const
{
  return speed;
}

void PlayerState::setSpeed(const TVector &value)
{
  speed = value;
}

double PlayerState::getDirection() const
{
  return direction;
}

TVector PlayerState::getDirectionVector() const
{
  return TVector::TPolarVector(1, direction);
}

void PlayerState::setDirection(double value)
{
  direction = value;
}

double PlayerState::getNeckDirection() const
{
  return neckDirection;
}

TVector PlayerState::getNeckDirectionVector() const
{
  return TVector::TPolarVector(1, neckDirection);
}

void PlayerState::setNeckDirection(double value)
{
  neckDirection = value;
}

int PlayerState::getUniformNumber() const
{
  return uniformNumber;
}

void PlayerState::setUniformNumber(int value)
{
  uniformNumber = value;
}

Intention* PlayerState::getIntention() const
{
  return intention;
}

void PlayerState::setIntention(Intention* _intention)
{
  intention = _intention;
}

double PlayerState::getC() const
{
  //  return min(position.getC(), speed.getC());
  return position.getC();
}

void PlayerState::setC(double _c)
{
  position.setC(_c);
  //  speed.setC(min(_c, speed.getC()));
  speed.setC(_c);
}

int PlayerState::getGameTime() const
{
  return gameTime;
}

void PlayerState::setGameTime(int _gameTime)
{
  gameTime = _gameTime;
}

TAngle PlayerState::getActualTurn(TAngle angle)
{
  return angle
    / (1.0 + parameterView->SP_inertia_moment * speed.getLength());
}

void PlayerState::step()
{
  // Advences the playerstate one cycle.
  TVector a = TVector(0,0,1);
  if(nextCommand == cmdDash)
    {
      //      cout << "Dash: " << dashPower << endl;
      a = getDirectionVector()
	* dashPower 
	* parameterView->SP_dash_power_rate;

      if(a.getLength() > parameterView->SP_player_accel_max)
	a.setLength(parameterView->SP_player_accel_max);
    }

  if(nextCommand == cmdTurn)
    {
      //      cout << "Turn: " << turnAngle << endl;
      direction = direction + getActualTurn(turnAngle);
    }

  if(nextCommand2 == cmdTurnNeck)
    {
      neckDirection = neckDirection + turnNeckAngle;
    }

  TVector u = speed + a;

  if(u.getC() > 0.5)
    position = position + u;

  if(u.getLength() > parameterView->SP_player_speed_max)
    u.setLength(parameterView->SP_player_speed_max);

  speed = u * parameterView->SP_player_decay;

  lastCommand = nextCommand;
  nextCommand = cmdNull;
  nextCommand2 = cmdNull;
  nextCommand3 = cmdNull;

  position.setC(position.getC() * 0.9);
  speed.setC(speed.getC() * 0.9);

  gameTime++;
}

PlayerState::cmdType PlayerState::getLastCommandType() const {
  return lastCommand;
}

// Returns number of chars needed.
std::string PlayerState::serialize() const
{
  std::string result; 
  short Short; //char Char; int Int; float Float;

  result += asString(Short = (short) (32000.0 * position.getX() / 60.0));
  result += asString(Short = (short) (32000.0 * position.getY() / 40.0));
  result += asString(Short = (short) (32000.0 * position.getC()));

  result += asString(Short = uniformNumber);

  return result;
}

// Returns number of chars consumed.
std::string PlayerState::deserialize(const std::string& buf)
{
  std::string result = buf; short Short = 0; // char Char; int Int;
  float _x, _y, _c;

  _x = fromString(Short, result) * 60.0 / 32000.0;
  result = skipString(Short, result);
  _y = fromString(Short, result) * 40.0 / 32000.0;
  result = skipString(Short, result);
  _c = fromString(Short, result) / 32000.0;
  result = skipString(Short, result);
  position = TVector(_x, _y, _c);

  uniformNumber = fromString(Short, result);
  result = skipString(Short, result);

  //  cout << "des-posC: " << position.getC() << endl;
  setC(0.85);

  nextCommand = cmdNull;
  nextCommand2 = cmdNull;

  return result;
}

// AgentState
AgentState::AgentState() : 
  PlayerState(),
  stamina(3500.0),
  effort(1.0),
  recovery(1.0)
{
}

AgentState::~AgentState()
{
  // Intentionally left blank.
}

float AgentState::getStamina() const
{
  return stamina;
}

void AgentState::setStamina(float value)
{
  stamina = value;
}

float AgentState::getEffort() const
{
  return effort;
}

void AgentState::setEffort(float value)
{
  effort = value;
}

float AgentState::getRecovery() const
{
  return recovery;
}

void AgentState::setRecovery(float value)
{
  recovery = value;
}

bool AgentState::operator==(const AgentState& other) const
{
  return
    recovery == other.recovery
    && stamina == other.stamina
    && effort == other.effort
    && PlayerState::operator==(other);
}

void AgentState::step()
{

  // Step stamina and stuff here...
  if(nextCommand == cmdDash)
    {
      if(dashPower > 0)
      {
	if(stamina < dashPower)
	  dashPower = stamina;
      }
      else
      {
	if(stamina < -dashPower * 2)
	  dashPower = stamina / 2;
      }

      if(dashPower > 0)
	stamina = stamina - dashPower;
      else
	stamina = stamina + 2 * dashPower;

      if(stamina < 0)
	stamina = 0;
    }

  // Step stamina.
  stamina += recovery * parameterView->SP_stamina_inc_max;
  if(stamina > parameterView->SP_stamina_max)
    stamina = parameterView->SP_stamina_max;

  // Step effort.
  if(stamina < (parameterView->SP_effort_dec_thr
		* parameterView->SP_stamina_max))
    {
      effort = effort - parameterView->SP_effort_dec;
      if(effort < parameterView->SP_effort_min)
	effort = parameterView->SP_effort_min;
    }
  else if(stamina > (parameterView->SP_effort_inc_thr
		     * parameterView->SP_stamina_max))
    {
      effort = effort + parameterView->SP_effort_inc;
      if(effort > 1.0)
	effort = 1.0;
    } 

  // Step Recovery.

  if(stamina < (parameterView->SP_recover_dec_thr
		* parameterView->SP_stamina_max))
    {
      recovery = recovery - parameterView->SP_recover_dec;
      if(recovery < parameterView->SP_recover_min)
	recovery = parameterView->SP_recover_min;
    }

  float effectivePower = dashPower * effort;
  if(stamina < dashPower)
    effectivePower = stamina * effort;
  dashPower = effectivePower;

  PlayerState::step();
}

// Handles assignment from PlayerStates.
const AgentState& AgentState::operator=(const PlayerState& other)
{
  stamina = parameterView->SP_stamina_max;
  recovery = 1.0;
  effort = 1.0;
  PlayerState::operator=(other);
  return *this;
}

// Returns number of chars needed.
std::string AgentState::serialize() const
{
  std::string result; // short Short; char Char; int Int; 
  float Float;
  result += asString(Float = stamina);
  result += asString(Float = effort);
  result += asString(Float = recovery);
  return result + PlayerState::serialize();
}

// Returns number of chars consumed.
std::string AgentState::deserialize(const std::string& buf)
{
  std::string result = buf; // short Short; char Char; int Int; 
  float Float = 0.0;

  //  cout << "AS-des" << endl;

  stamina = fromString(Float, result);
  result = skipString(Float, result);

  effort = fromString(Float, result);
  result = skipString(Float, result);

  recovery = fromString(Float, result);
  result = skipString(Float, result);

  return PlayerState::deserialize(result);
}

RS_END_NAMESPACE
