/* -*- Mode: C++ -*-
 * $Id: WithUC.h,v 1.6 2001/09/01 23:53:09 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WithUC.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/09/01 23:53:09 $
 *
 *
 * HISTORY
 *
 * $Log: WithUC.h,v $
 * Revision 1.6  2001/09/01 23:53:09  frehe
 * Changed the implementation of getValue and getAngle.
 *
 * Revision 1.5  2001/08/29 21:08:29  frehe
 * Replaces class T with typename T where T can be a built in type.
 *
 * Revision 1.4  2000/09/03 14:23:20  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.3  2000/09/01 11:43:02  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 03:23:54  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:43  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 *
 */

/**
 * @file WithUC.h
 * Include file for the template WithUC.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:53:09 $
 */


#ifndef WITHUC_H
#define WITHUC_H


#include "WithUnknown.h"
#include "WithConfidence.h"


RS_BEGIN_NAMESPACE


/**
 * @class WithUC WithUC.h
 * Template class for representing basic types that can be both unknown
 * and have a confidence value.
 * The only demands on the template argument T is that the arithmetic
 * functions are defined on it.
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:53:09 $
 */
template<typename T>
class WithUC : public WithUnknown<T> {
public:
  WithUC();
  WithUC(const T& v,
	 const Confidence& conf = Confidence(),
	 const bool print_conf = true);
  WithUC(const WithUnknown<T>& wu,
	 const Confidence& conf = Confidence(),
	 const bool print_conf = true);
  WithUC(const WithConfidence<T>& wc);
  WithUC(const WithUC<T>& wuc);

  //WithUnknown<T> getValue() const;

  Confidence getConfidence() const;
  void setConfidence(const Confidence& conf);

  bool printConfidence() const;
  void setPrintConfidence(const bool print_confidence);
  
  WithUC<T>& operator= (const WithUC<T>& op);

  WithUC<T>& operator+= (const WithUC<T>& op);
  WithUC<T>& operator-= (const WithUC<T>& op);
  WithUC<T>& operator*= (const WithUC<T>& op);
  WithUC<T>& operator/= (const WithUC<T>& op);

  WithUC<T> operator- () const;
  WithUC<T> operator+ (const WithUC<T>& op) const;
  WithUC<T> operator- (const WithUC<T>& op) const;
  WithUC<T> operator* (const WithUC<T>& op) const;
  WithUC<T> operator/ (const WithUC<T>& op) const;
  
private:
  Confidence confidence;
  bool doPrintConfidence;
};

template<typename T>
WithUC<T>::WithUC()
  : WithUnknown<T>(),
    confidence(),
    doPrintConfidence(true)
{
}

template<typename T>
WithUC<T>::WithUC(const T& v, const Confidence& conf, const bool print_conf)
  : WithUnknown<T>(v),
    confidence(conf),
    doPrintConfidence(print_conf)
{
}

template<typename T>
WithUC<T>::WithUC(const WithUnknown<T>& wu,
		  const Confidence& conf,
		  const bool print_conf)
  : WithUnknown<T>(wu),
    confidence(conf),
    doPrintConfidence(print_conf)
{
}

template<typename T>
WithUC<T>::WithUC(const WithConfidence<T>& wc)
  : WithUnknown<T>(wc.getValue()),
    confidence(wc.getConfidence()),
    doPrintConfidence(wc.doPrintConfidence())
{
}

template<typename T>
WithUC<T>::WithUC(const WithUC<T>& wuc)
  : WithUnknown<T>(wuc),
    confidence(wuc.confidence),
    doPrintConfidence(wuc.doPrintConfidence)
{
}

/*
template<typename T>
inline WithUnknown<T> WithUC<T>::getValue() const
{
  return WithUnknown<T>(*this);
}
*/

template<typename T>
inline Confidence WithUC<T>::getConfidence() const
{
  return confidence;
}

template<typename T>
inline void WithUC<T>::setConfidence(const Confidence& conf)
{
  confidence = conf;
}

template<typename T>
inline bool WithUC<T>::printConfidence() const
{
  return doPrintConfidence;
}

template<typename T>
inline void WithUC<T>::setPrintConfidence(const bool print_conf)
{
  doPrintConfidence = print_conf;
}

template<typename T>
WithUC<T>& WithUC<T>::operator= (const WithUC<T>& op)
{
  if ( &op != this ) {
    WithUnknown<T>::operator=(op);
    confidence = op.confidence;
    doPrintConfidence = op.doPrintConfidence;
  }

  return *this;
}

template<typename T>
WithUC<T>& WithUC<T>::operator+= (const WithUC<T>& op)
{
  WithUnknown<T>::operator+=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
WithUC<T>& WithUC<T>::operator-= (const WithUC<T>& op)
{
  WithUnknown<T>::operator-=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
WithUC<T>& WithUC<T>::operator*= (const WithUC<T>& op)
{
  WithUnknown<T>::operator*=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
WithUC<T>& WithUC<T>::operator/= (const WithUC<T>& op)
{
  WithUnknown<T>::operator/=(op);
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  confidence = min(confidence, op.confidence);
#else
  confidence = max(confidence, op.confidence);
#endif
  return *this;
}

template<typename T>
inline WithUC<T> WithUC<T>::operator- () const
{
  return WithUC<T>(WithUnknown<T>::operator-(),
			   confidence, doPrintConfidence);
}

template<typename T>
inline WithUC<T> WithUC<T>::operator+ (const WithUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(WithUnknown<T>::operator+(op),
			   min(confidence, op.confidence),
			   doPrintConfidence);
#else
  return WithUC<T>(WithUnknown<T>::operator+(op),
			   max(confidence, op.confidence),
			   doPrintConfidence);
#endif
}

template<typename T>
inline WithUC<T> WithUC<T>::operator- (const WithUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(WithUnknown<T>::operator-(op),
			   min(confidence, op.confidence),
			   doPrintConfidence);
#else
  return WithUC<T>(WithUnknown<T>::operator-(op),
			   max(confidence, op.confidence),
			   doPrintConfidence);
#endif
}

template<typename T>
inline WithUC<T> WithUC<T>::operator* (const WithUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(WithUnknown<T>::operator*(op),
			   min(confidence, op.confidence),
			   doPrintConfidence);
#else
  return WithUC<T>(WithUnknown<T>::operator*(op),
			   max(confidence, op.confidence),
			   doPrintConfidence);
#endif
}

template<typename T>
inline WithUC<T> WithUC<T>::operator/ (const WithUC<T>& op) const
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(WithUnknown<T>::operator/(op),
		   min(confidence, op.confidence),
		   doPrintConfidence);
#else
  return WithUC<T>(WithUnknown<T>::operator/(op),
		   max(confidence, op.confidence),
		   doPrintConfidence);
#endif
}

template<typename T>
inline WithUC<T> operator+ (const T& op1, const WithUC<T>& op2)
{
  return op2 + op1;
}

template<typename T>
inline WithUC<T> operator- (const T& op1, const WithUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(op1 - WithUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.doPrintConfidence());
#else
  return WithUC<T>(op1 - WithUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.doPrintConfidence());
#endif
}

template<typename T>
inline WithUC<T> operator* (const T& op1, const WithUC<T>& op2)
{
  return op2 * op1;
}

template<typename T>
inline WithUC<T> operator/ (const T& op1, const WithUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(op1 / WithUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return WithUC<T>(op1 / WithUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline WithUC<T> operator+ (const WithUnknown<T>& op1, const WithUC<T>& op2)
{
  return op2 + op1;
}

template<typename T>
inline WithUC<T> operator- (const WithUnknown<T>& op1, const WithUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(op1 - WithUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return WithUC<T>(op1 - WithUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline WithUC<T> operator* (const WithUnknown<T>& op1, const WithUC<T>& op2)
{
  return op2 * op1;
}

template<typename T>
inline WithUC<T> operator/ (const WithUnknown<T>& op1, const WithUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(op1 / WithUnknown<T>(op2),
		   min(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return WithUC<T>(op1 / WithUnknown<T>(op2),
		   max(Confidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline WithUC<T> operator+ (const WithConfidence<T>& op1, const WithUC<T>& op2)
{
  return op2 + op1;
}

template<typename T>
inline WithUC<T> operator- (const WithConfidence<T>& op1, const WithUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(op1 - WithUnknown<T>(op2),
		   min(op1.getConfidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return WithUC<T>(op1 - WithUnknown<T>(op2),
		   max(op.getConfidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
inline WithUC<T> operator* (const WithConfidence<T>& op1, const WithUC<T>& op2)
{
  return op2 * op1;
}

template<typename T>
inline WithUC<T> operator/ (const WithConfidence<T>& op1, const WithUC<T>& op2)
{
#ifdef RS_PESSIMISTIC_OBSERVATIONS
  return WithUC<T>(op1 / WithUnknown<T>(op2),
		   min(getConfidence(), op2.getConfidence()),
		   op2.printConfidence());
#else
  return WithUC<T>(op1 / WithUnknown<T>(op2),
		   max(getConfidence(), op2.getConfidence()),
		   op2.printConfidence());
#endif
}

template<typename T>
std::ostream& operator<<(std::ostream& os, const WithUC<T>& obj)
{
  if (obj.printConfidence())
    return os << WithUnknown<T>(obj) << " " << obj.getConfidence();
  else
    return os << WithUnknown<T>(obj);
}

template<typename T>
WithUC<T> sqrt(const WithUC<T>& op)
{
  return WithUC<T>(sqrt(WithUnknown<T>(op)));
}


RS_END_NAMESPACE


#endif // !WITHUC_H
