/* -*- Mode: C++ -*-
 * $Id: rs_try_catch.h,v 1.1 2001/09/03 19:38:42 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: rs_throw.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2001-09-03
 * Last modified by $Author: frehe $ $Date: 2001/09/03 19:38:42 $
 *
 *
 * HISTORY
 *
 * $Log: rs_try_catch.h,v $
 * Revision 1.1  2001/09/03 19:38:42  frehe
 * First version of the file.
 *
 */

/**
 * @file rs_try_catch.h
 * Include file for the RoboSoc RS_BEGIN_TRY, RS_END_TRY and RS_CATCH
 * exception catching macros.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2001/09/03 19:38:42 $
 */


#ifndef RS_TRY_CATCH_H
#define RS_TRY_CATCH_H


#include "library_compiler_directives.h"


/** @def RS_BEGIN_TRY
    This macro is used to begin a try { ... } catch () section. */
/** @def RS_END_TRY
    This macro is used to end a try { ... } catch () section. */
/** @def RS_CATCH( Exception, Code )
    This macro is used to create a catch section after a try section.
    @a Exception should be the declaration of the variable of the exception
    type to catch, e.g. RSException& excpt. @a Code should be the code that
    should be executed when the exception is caught. */
#ifdef RS_USE_EXCEPTION

#  define RS_BEGIN_TRY try {
#  define RS_END_TRY }
#  define RS_CATCH( Exception, Code ) catch ( Exception ) { Code }


#else // !RS_USE_EXCEPTION

#  define RS_BEGIN_TRY
#  define RS_END_TRY
#  define RS_CATCH( Exception, Code )

#endif // RS_USE_EXCEPTION


#endif // !RS_THROW_H
