/* -*- Mode: C++ -*-
 * $Id: FuzzyBool.h,v 1.5 2001/09/29 18:54:53 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FuzzyBool.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/09/29 18:54:53 $
 *
 *
 * HISTORY
 *
 * $Log: FuzzyBool.h,v $
 * Revision 1.5  2001/09/29 18:54:53  frehe
 * Remove TRUE, FALSE and UNKNOWN.
 *
 * Revision 1.4  2001/09/26 11:40:41  frehe
 * Moved TRUE, FALSE and UNKNOWN inside FuzzyBool.
 *
 * Revision 1.3  2000/09/17 15:36:57  frehe
 * Added the UNKNOWN constant to FuzzyBool, it represents a value that is neither true nor false.
 *
 * Revision 1.2  2000/09/05 11:07:24  frehe
 * Added the constants TRUE and FALSE.
 *
 * Revision 1.1  2000/09/01 11:43:40  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FuzzyBool.h
 * Include file for the class FuzzyBool.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2001/09/29 18:54:53 $
 */


#ifndef FUZZYBOOL_H
#define FUZZYBOOL_H


#include "library_types.h"
#include "math_utils.h"


RS_BEGIN_NAMESPACE


/**
 * @class FuzzyBool FuzzyBool.h
 * Class used to represent truth values that can have
 * a value between 0 and 1, where 0 is false and 1 is true.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2001/09/29 18:54:53 $
 */
class FyzzyBool;
class FuzzyBool {
public:
  /** <description of this FuzzyBool constructor> */
  FuzzyBool(const Float val = 0);
  FuzzyBool(const bool val);
  FuzzyBool(const FuzzyBool& obj);

  FuzzyBool operator!() const;
  FuzzyBool operator&&(const FuzzyBool& op) const;
  FuzzyBool operator||(const FuzzyBool& op) const;

  bool operator<(const FuzzyBool& op) const;
  bool operator>(const FuzzyBool& op) const;

  operator bool() const;
  
  Float getValue() const;
  
  bool isTrue() const;
  bool isFalse() const;


private:
  Float value;
};


/*
// Constant representing a true FuzzyBool.
const FuzzyBool TRUE = FuzzyBool(1.0);
// Constant representing a false FuzzyBool.
const FuzzyBool FALSE = FuzzyBool(0.0);
// Constant representing an unknown FuzzyBool.
const FuzzyBool UNKNOWN = FuzzyBool(0.5);
*/

  
std::ostream& operator<<(std::ostream& os, const FuzzyBool& f);

  
RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "FuzzyBool.icc"
#endif


#endif // !FUZZYBOOL_H
